package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.entity.BaseDieselGeneratorEnt;
import pps.core.base.entity.BasePowerLineDieselEnt;
import pps.core.base.mapper.BaseDieselGeneratorMapper;
import pps.core.base.mapper.BasePowerLineDieselMapper;
import pps.core.base.service.data.base_diesel_generator.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;
import java.util.Objects;

/**
 * 柴发设备模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseDieselGeneratorService extends BaseService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseDieselGenerator(XContext context, CreateBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        BaseDieselGeneratorEnt entity = new BaseDieselGeneratorEnt();
        XCopyUtils.copyObject(input, entity);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseDieselGenerator(XContext context, UpdateBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseDieselGeneratorEnt::getId, input.getId())
                .eq(BaseDieselGeneratorEnt::getIsDeleted, BusinessConstant.ONE);
        BaseDieselGeneratorEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseDieselGenerator(XContext context, DeleteBaseDieselGeneratorInput input) {
        String dieselId = input.getId();
        if (this.checkDieselIsReference(context, dieselId)) {
            return XServiceResult.error(992, "当前柴发设备已被引用");
        }
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseDieselGeneratorEnt::getId, input.getId())
                .eq(BaseDieselGeneratorEnt::getIsDeleted, BusinessConstant.ONE);
        BaseDieselGeneratorEnt entity = mapper.selectOne(queryWrapper);
        if (Objects.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity = new BaseDieselGeneratorEnt();
        entity.setId(input.getId());
        entity.setIsDeleted(BusinessConstant.ZERO);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseDieselGeneratorOutput> getBaseDieselGenerator(XContext context, GetBaseDieselGeneratorInput input) {
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseDieselGeneratorEnt::getId, input.getId())
                .eq(BaseDieselGeneratorEnt::getIsDeleted, BusinessConstant.ONE);
        BaseDieselGeneratorEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDieselGeneratorOutput output = new GetBaseDieselGeneratorOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDieselGeneratorOutput> queryBaseDieselGenerator(XContext context, QueryBaseDieselGeneratorInput input) {
        String ouId = input.getOuId();
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        String excludeIdFlag = input.getExcludeIdFlag();
        BaseDieselGeneratorMapper mapper = context.getBean(BaseDieselGeneratorMapper.class);
        List<String> allListByOuId = this.getAllListByOuId(context, input.getOuId());
        QueryWrapper<BaseDieselGeneratorEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .notExists("1".equals(excludeIdFlag), "SELECT 1 FROM base_power_line_diesel w WHERE w.ou_id = {0} AND base_diesel_generator.id = w.diesel_id", input.getOuId())
                .eq(BaseDieselGeneratorEnt::getIsDeleted, BusinessConstant.ONE)
                .in(CollUtil.isNotEmpty(allListByOuId), BaseDieselGeneratorEnt::getOuId, allListByOuId)
                .like(StrUtil.isNotEmpty(ouName), BaseDieselGeneratorEnt::getOuName, ouName)
                .like(StrUtil.isNotEmpty(deviceName), BaseDieselGeneratorEnt::getDeviceName, deviceName)
                .orderByDesc(BaseDieselGeneratorEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDieselGeneratorOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查柴发设备是否被引用
     *
     * @param context  上下文
     * @param dieselId 柴发设备ID
     * @return boolean
     */
    private boolean checkDieselIsReference(XContext context, String dieselId) {
        BasePowerLineDieselMapper mapper = context.getBean(BasePowerLineDieselMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineDieselEnt>()
                .eq(BasePowerLineDieselEnt::getDieselId, dieselId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }
}
