package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.entity.BaseEnergyStorageDeviceEnt;
import pps.core.base.entity.BasePowerLineStorageEnt;
import pps.core.base.mapper.BaseEnergyStorageDeviceMapper;
import pps.core.base.mapper.BasePowerLineStorageMapper;
import pps.core.base.service.data.base_energy_storage_device.*;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;
import java.util.Objects;

/**
 * 储能设备模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseEnergyStorageDeviceService extends BaseService {

    @XText("新增")
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseEnergyStorageDevice(XContext context, CreateBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        BaseEnergyStorageDeviceEnt entity = new BaseEnergyStorageDeviceEnt();
        XCopyUtils.copyObject(input, entity);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XText("更新")
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseEnergyStorageDevice(XContext context, UpdateBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getId, input.getId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, BusinessConstant.ONE);
        BaseEnergyStorageDeviceEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除")
    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseEnergyStorageDevice(XContext context, DeleteBaseEnergyStorageDeviceInput input) {
        String storageId = input.getId();
        if (this.checkStorageIsReference(context, storageId)) {
            return XServiceResult.error(992, "当前储能设备已被引用");
        }
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getId, input.getId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, BusinessConstant.ONE);
        BaseEnergyStorageDeviceEnt entity = mapper.selectOne(queryWrapper);
        if (Objects.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity = new BaseEnergyStorageDeviceEnt();
        entity.setId(input.getId());
        entity.setIsDeleted(BusinessConstant.ZERO);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        BaseUtils.setBaseModelDefault(entity, session);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("根据id获取详情")
    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseEnergyStorageDeviceOutput> getBaseEnergyStorageDevice(XContext context, GetBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getId, input.getId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, BusinessConstant.ONE);
        BaseEnergyStorageDeviceEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEnergyStorageDeviceOutput output = new GetBaseEnergyStorageDeviceOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XText("分页查询")
    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseEnergyStorageDeviceOutput> queryBaseEnergyStorageDevice(XContext context, QueryBaseEnergyStorageDeviceInput input) {
        String ouId = input.getOuId();
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        String excludeIdFlag = input.getExcludeIdFlag();
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        List<String> allListByOuId = this.getAllListByOuId(context, input.getOuId());
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .notExists("1".equals(excludeIdFlag), "SELECT 1 FROM base_power_line_storage w WHERE w.ou_id = {0} AND base_energy_storage_device.id = w.storage_id", input.getOuId())
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, BusinessConstant.ONE)
                .in(CollUtil.isNotEmpty(allListByOuId), BaseEnergyStorageDeviceEnt::getOuId, allListByOuId)
                .like(StrUtil.isNotEmpty(ouName), BaseEnergyStorageDeviceEnt::getOuName, ouName)
                .like(StrUtil.isNotEmpty(deviceName), BaseEnergyStorageDeviceEnt::getDeviceName, deviceName)
                .orderByDesc(BaseEnergyStorageDeviceEnt::getModifyTime);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEnergyStorageDeviceOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查储能设备是否被引用
     *
     * @param context   上下文
     * @param storageId 储能设备ID
     * @return boolean
     */
    private boolean checkStorageIsReference(XContext context, String storageId) {
        BasePowerLineStorageMapper mapper = context.getBean(BasePowerLineStorageMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                .eq(BasePowerLineStorageEnt::getStorageId, storageId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }
}